module Push
  module Huawei
    ACCESS_TOKEN_KEY = "push:huawei_push:access_token".freeze
    TOKEN_URL = 'https://login.cloud.huawei.com/oauth2/v2/token'.freeze
    PUSH_URL = 'https://api.push.hicloud.com/pushsend.do'.freeze
    ACCESS_TOEKN_EXPIRE_TIME = 3500
    BATCH_PUSH_MAX = 80

    class << self
      # 4a7d376b47c847634571a86b37e93272 测试使用的device_token
      def push(device_tokens, message)
        handle_token_res if access_token.blank?
        device_tokens.each_slice(BATCH_PUSH_MAX) do |device_token_array|
          res_push(message, device_token_array)
        end
      rescue StandardError => err
        Push::Log.error(err)
      end

      def res_push(message, device_tokens)
        body = message(message, device_tokens)

        nsp_ctx = {
          ver: '1',
          appId: PushSettings.huawei['app_id']
        }
        url = PUSH_URL + "?nsp_ctx=#{CGI::escape(nsp_ctx.to_json)}"
        post(url, body)
      end

      # 推送信息
      def message(message, device_tokens)
        payload = {
          hps: {
            msg: {
              type: 1,
              body: message
            }
          }
        }
        Push::Log.info("huawei_payload: #{payload}, device_tokens: #{device_tokens}")
        params = {
          access_token: CGI.escape(access_token),
          nsp_svc: CGI.escape('openpush.message.api.send'),
          nsp_ts: CGI.escape(Time.now.to_i.to_s),
          device_token_list: CGI.escape(device_tokens.to_s),
          payload: CGI.escape(payload.to_json)
        }
        params_str = ''
        params.each_with_index do |(key,value), index|
          params_str += "#{key}=#{value}"
          params_str += '&' if index + 1 < params.size
        end
        params_str
      end

      def handle_token_res
        loop do
          res = req_token
          if res['error'].blank? && res['access_token'].present?
            $redis.set(ACCESS_TOKEN_KEY, res['access_token'], ex: ACCESS_TOEKN_EXPIRE_TIME)
            break
          end
        end
      end

      def req_token
        body = {
          grant_type: 'client_credentials',
          client_id: PushSettings.huawei['app_id'],
          client_secret: PushSettings.huawei['app_secret']
        }
        post(TOKEN_URL, body)
      end

      def access_token
        $redis.get(ACCESS_TOKEN_KEY)
      end

      def post(url, body)
        headers = { 'Content-Type' => 'application/x-www-form-urlencoded' }
        response = HTTParty.post(url, body: body, headers: headers, timeout: 5)
        Push::Log.info("huawei_push: \n access_token: #{access_token} expire_time: #{$redis.ttl(Push::Huawei::ACCESS_TOKEN_KEY)}\n  url: #{url}\n body: #{body}\n response: #{response}")
        JSON.parse(response)
      end
    end
  end
end