module Push
  module Igetui
    class << self

      def push(user_device_ids, template_data = {}, opts = {})
        # 包装成igetui 真正需要的参数格式
        opts[:transmission_content] = template_data
        if user_device_ids.size > 1
          push_transmission_to_list_igetui(user_device_ids, opts)
        else
          push_transmission_to_igetui(user_device_ids, opts)
        end
      rescue StandardError => err
        ::Push::Log.error(err)
      end

      def config
        {
          push_type: 2
        }
      end

      def push_transmission_to_igetui(user_device_id, template_data = {})
        template_data = handle_template_data(template_data)

        push_igetui_with_asnyc(
          user_device_id,
          template_data,
          template_type: 'TransmissionTemplate'
        )
      end

      def push_transmission_to_list_igetui(user_device_ids, template_data = {})
        template_data = handle_template_data(template_data)
        push_list_igetui_with_asnyc(
          user_device_ids,
          template_data,
          template_type: 'TransmissionTemplate', pusher_type: 'push_message_to_list'
        )
      end

      private

      def handle_template_data(template_data)
        template_data = HashWithIndifferentAccess.new(template_data)
        transmission_content = HashWithIndifferentAccess.new(template_data.delete(:transmission_content))
        transmission_content[:push_type] ||= config[:push_type]
        template_data[:transmission_type] ||= 0
        template_data[:transmission_content] = transmission_content.as_json
        template_data
      end

      def push_igetui_with_asnyc(user_device_id, template_data = {}, opts = nil)
        return if template_data.blank?

        opts ||= HashWithIndifferentAccess.new(template_type: 'TransmissionTemplate')

        template_data = HashWithIndifferentAccess.new(template_data)
        perform(template_data, opts, user_device_id)
      end

      def push_list_igetui_with_asnyc(user_device_ids, template_data = {}, opts = { pusher_type: 'push_message_to_list' })
        return if user_device_ids.blank? || template_data.blank?

        opts ||= HashWithIndifferentAccess.new(template_type: 'TransmissionTemplate')

        perform(template_data, opts, user_device_ids)
      end

      def perform(template_data, opts, user_device_ids)
        opts = HashWithIndifferentAccess.new(opts)
        igetui = Igetui::Push.new(template_data, opts)
        result = igetui.push_message_to(user_device_ids)
        ::Push::Log.info("igetui_push: \n template_data: #{template_data}\n opts: #{opts}\n user_device_ids: #{user_device_ids}\n response: #{result}")
      end
    end
  end
end